#!/usr/bin/perl

use Cwd;

$hlen=512;
$flen=16384*3+$hlen;
$tlen=$flen*32;
$maxfile=0;

print "\n##############################################################################################\n";
print "#                                                                                            #\n";
print "#  AX82 z80 file to image converter V0.35                                                    #\n";
print "#                                                                                            #\n";
print "#  (c) 2012-2014 Joerg Wolfram                                                               #\n";
print "#                                                                                            #\n";
print "##############################################################################################\n";

#add .z80 files
$muster="*.z80";
@fnames1=<${muster}>;
$tmaxfile=$#fnames1+1;

print "Files z80 = $tmaxfile\n";

if($tmaxfile>0)
{
	for($i=0;$i<=$tmaxfile;$i++)
	{
		if (length($fnames1[$i])>0)
		{
			$fnames[$maxfile++]=$fnames1[$i];
		}
	}
}

#add .Z80 files
$muster="*.Z80";
@fnames2=<${muster}>;
$tmaxfile=$#fnames2+1;

print "Files Z80 = $tmaxfile\n";

if($tmaxfile>0)
{
	for($i=0;$i<=$tmaxfile;$i++)
	{
		if (length($fnames2[$i])>0)
		{
			$fnames[$maxfile++]=$fnames2[$i];
		}
	}
}

@fnames=sort(@fnames);

$dir=getcwd;
$dir =~ /TAPE_/;
$tnumber=$';

if (length($tnumber) != 3)
{
	die "wrong place\n";
}
$tnumber=$tnumber+0;

#format tape (clear all entries)
print "** format tape $tnumber\n";
open(BWRITE,"+<../../ax82_img.bin");
binmode(BWRITE);
$offset=512+$tlen*$tnumber;
seek(BWRITE,$offset,0);

for($i=0;$i<$tlen;$i++)
{
	print BWRITE chr(255);
}
close(BWRITE);

# limit to 32 files max
if ($maxfile > 32)
{
	$maxfile = 31;
}

for($k=0;$k<$maxfile;$k++)
{
	$infile=$fnames[$k];
	$fname=$fnames[$k];
	$fname=~ /\./;
	$fname=$`;
	$pinfile=substr($infile."                         ",0,16);

	print "processing $pinfile ";

	$dlen= -s $infile;

	$foffset=$offset+$flen*$k;

	open(BREAD,$infile);
	binmode(BREAD);

	$thisname=$fname;
	$fname.="          ";

	#clear head buffer
	for($i=0;$i<512;$i++)
	{
		$thead[$i]=255;
	}

	#clear file buffer
	for($i=0;$i<49152;$i++)
	{
		$tdata[$i]=255;
	}

	#copy file name
	for($i=0;$i<12;$i++)
	{
		$thead[$i]=ord(substr($fname,$i,1));
	}
	#set valid flag
	$thead[16]=0xcc;

	$ftype=1;

	#copy header 1
	$header_len=30;
	for($i=0;$i<30;$i++)
	{
		read(BREAD,$tbyte,1);
		$dlen--;
		$head1[$i]=ord($tbyte);
	}

	if(($head1[6]==0) && ($head1[7]==0))
	{
		print ">> MODE 2  ";
		read(BREAD,$tbyte,1);
		$dlen--;
		$head2len=ord($tbyte);
		read(BREAD,$tbyte,1);
		$dlen--;
		$head2len+=ord($tbyte)*256;
		
		if($head2len > 54)
		{
			$ftype=99;
			printf "$thisname has invalid format\n";
		}

		for($i=0;$i<$head2len;$i++)
		{
			read(BREAD,$tbyte,1);
			$dlen--;
			$head2[$i]=ord($tbyte);
		}

		#copy PC value from header2 to header1
		$head1[6]=$head2[0];
		$head1[7]=$head2[1];

		if($head2[4] > 1)
		{
			$ftype=99;
			printf "$thisname is not hardware compatible\n";
		}

		while($dlen>3)
		{
			read(BREAD,$tbyte,1);
			$dlen--;
			$blen=ord($tbyte);
			read(BREAD,$tbyte,1);
			$dlen--;
			$blen+=ord($tbyte)*256;
			read(BREAD,$tbyte,1);
			$dlen--;
			$page=ord($tbyte);
			$poffset=-1;
			if ($page==8) { $poffset=0; }
			if ($page==4) { $poffset=16384; }
			if ($page==5) { $poffset=32768; }
			if ($poffset >=0)
			{
				print ">> Page $page ";
				&uncompress_data($blen,$poffset,16384);
			}
			else
			{
				die("\nWrong page $page in $infile\n");
			}
		}
		print ">> End of file\n";
	}
	else
	{
		if(($head1[12] & 32) == 32)
		{
			print ">> MODE 1C ";
			&uncompress_data($dlen-4,0,49152);
		print ">> End of file, rest $dlen\n";

		}
		else
		{
			print ">> MODE 1U ";
			&copy_data(49152,0);
			print ">> End of file, rest $dlen\n";
		}
	}

	close(BREAD);

	open(BWRITE,"+<../../ax82_img.bin");
	binmode(BWRITE);
	seek(BWRITE,$foffset,0);
	for($i=0;$i<256;$i++)
	{
		print BWRITE chr($thead[$i]);
	}
	for($i=0;$i<30;$i++)
	{
		print BWRITE chr($head1[$i]);
	}
	for($i=0;$i<226;$i++)
	{
		print BWRITE chr(255);
	}

	for($i=0;$i<49152;$i++)
	{
		print BWRITE chr($tdata[$i]);
	}
	close(BWRITE);
}
print "** DONE ***\n";


sub copy_data
{
	my $blocklen=$_[0];
	my $dstart=$_[1];
	my $idx=0;
	for($idx=0;$idx<$blocklen;$idx++)
	{
		read(BREAD,$tbyte,1);
		$dlen--;
		$tdata[$idx+$dstart]=ord($tbyte);
	}
}

sub uncompress_data
{
	my $blocklen=$_[0];
	my $dstart=$_[1];
	my $pagelen=$_[2];
	my $idx=0;
	my $count=0;
	my $pos=$dstart;
	my $ed1=0;
	my $ed2=0;
	my $crep=0;
	my $cval=0;
	my $do_next=1;

	while($blocklen > 0)
	{
		read(BREAD,$tbyte,1);
		$dlen--;
		$blocklen--;
		$this_byte=ord($tbyte);
		$ed1=$ed2;
		$ed2=$crep;
		$crep=$cval;
		$cval=$this_byte;
		$crep1=$crep;
		if($crep1 == 0)
		{
			$crep1=256;
		}
		
		if(($ed1 == 0xed) && ($ed2 == 0xed) && ($crep1 > 4))
		{
			if($crep == 0)
			{
				print "256\n";
			}
			
			$pos-=3;	#discard last three bytes
			for($count=0;$count<$crep1;$count++)
			{
				if (($pos-$dstart) < $pagelen)
				{
					$tdata[$pos++]=$cval;
				}
				else
				{
					print "Out of block ";
				}
			}
			$ed1=0;
			$ed2=0;
			$crep=0;
			$cval=0;
		}
		else
		{
			$tdata[$pos++]=$this_byte;
		}
	}
}
